@@ -439,7 +439,7 @@ def plotSignatureBasedAverageNucleosomeOccupancyFigureWithSimulations(sample,sig
         # if (np.argwhere(np.isnan(realAverage))).size>0:
         if (np.argwhere(pd.isnull(realAverage))).size > 0:
             print('Attention: There are %d nans in realAverage in %s for %s' %(len(np.argwhere(np.isnan(realAverage))),libraryFilenameMemo,signature))
-            if verbose: print(np.argwhere(pd.isnull(realAverage)))
+            if verbose: print('\tVerbose %s' %(np.argwhere(pd.isnull(realAverage))))
 
     if ((realAverage is not None) and (pd.notna(realAverage).any(axis=0)) and (np.any(realAverage))):
         min_list.append(np.amin(realAverage))
@@ -479,9 +479,9 @@ def plotSignatureBasedAverageNucleosomeOccupancyFigureWithSimulations(sample,sig
             if (simulationsSignatureBasedLows is not None) and (simulationsSignatureBasedHighs is not None):
                 plt.fill_between(x, np.array(simulationsSignatureBasedLows), np.array(simulationsSignatureBasedHighs),facecolor=fillcolor)
 
-        if (simulationsSignatureBasedLows is not None and simulationsSignatureBasedLows):
+        if ((simulationsSignatureBasedLows is not None) and (not np.all(np.isnan(simulationsSignatureBasedLows)))):
             min_list.append(np.nanmin(simulationsSignatureBasedLows))
-        if (simulationsSignatureBasedHighs is not None and simulationsSignatureBasedHighs):
+        if ((simulationsSignatureBasedHighs is not None) and (not np.all(np.isnan(simulationsSignatureBasedHighs)))):
             max_list.append(np.nanmax(simulationsSignatureBasedHighs))
 
         plt.legend(loc= 'lower left', handles=listofLegends, prop={'size': 24}, shadow=False, edgecolor='white', facecolor='white',framealpha=0)
@@ -586,7 +586,6 @@ def plotSignatureBasedAverageNucleosomeOccupancyFigureWithSimulations(sample,sig
 ############################ Plot Figure ####################################
 #############################################################################
 def plotAllMutationsPooledWithSimulations(xlabel,ylabel,sample,outputDir,jobname,numberofSubs,numberofIndels,numberofDinucs,numberofSimulations,mutationType,libraryFilename,libraryFilenameMemo,occupancy_type,plusOrMinus):
-
     if mutationType==SBS96:
         to_be_added_to_the_filename='SBS%s' %(mutationType)
     else:
@@ -656,7 +655,6 @@ def plotAllMutationsPooledWithSimulations(xlabel,ylabel,sample,outputDir,jobname
                 listofSimulationsAggregatedDinucs = readDataForSimulations(sample, None, SAMPLEBASED_AGGREGATEDDINUCS, outputDir,jobname,numberofSimulations,occupancy_type,libraryFilenameMemo)
     #######################################################################################################################
 
-
     #####################################################################
     #95%CI
     simulationsAggregatedSubstitutionsLows,simulationsAggregatedSubstitutionsMedians,simulationsAggregatedSubstitutionsHighs = takeAverage(listofSimulationsAggregatedSubstitutions)
@@ -750,7 +748,6 @@ def plotAllMutationsPooledWithSimulations(xlabel,ylabel,sample,outputDir,jobname
     plt.text(0.99, 0.99, text, verticalalignment='top', horizontalalignment='right', transform=ax.transAxes, fontsize=24)
     #######################################################################
 
-
     #put the library filename
     plt.text(0.01, 0.99, libraryFilename, verticalalignment='top', horizontalalignment='left', transform=ax.transAxes,fontsize=24)
 
@@ -954,14 +951,14 @@ def updateDictWithThreeLevels(signature2Biosample2ENCODEHM2FoldChangeDict, signa
 
 
 ########################################################
-def calculate_fold_change(output_dir,numberofSimulations,signature,cancer_type,encode_hm):
+def calculate_fold_change(output_dir,numberofSimulations,signature,cancer_type,encode_hm,plusOrMinus_epigenomics,verbose):
     occupancy_type=EPIGENOMICSOCCUPANCY
-    center=2000
+    center=plusOrMinus_epigenomics
     plusorMinus=250
     start=center-plusorMinus
     end= center+plusorMinus+1
 
-    print('----------> %s %s %s' % (signature, cancer_type, encode_hm))
+    if verbose: print('\tVerbose ----------> %s %s %s' % (signature, cancer_type, encode_hm))
     avg_real_signal = None
 
     # SBS1_sim1_ENCFF330CCJ_osteoblast_H3K79me2-human_AverageSignalArray.txt
@@ -973,7 +970,8 @@ def calculate_fold_change(output_dir,numberofSimulations,signature,cancer_type,e
         # avg_real_data_signal_array[np.isnan(avg_real_data_signal_array)] = 0
         # avg_real_signal = np.mean(avg_real_data_signal_array[1750:2251])
         # 2nd way
-        avg_real_signal = np.nanmean(avg_real_data_signal_array[start:end])
+        if not np.all(np.isnan(avg_real_data_signal_array[start:end])):
+            avg_real_signal = np.nanmean(avg_real_data_signal_array[start:end])
 
     avg_simulated_signal = None
     if (numberofSimulations > 0):
@@ -982,27 +980,30 @@ def calculate_fold_change(output_dir,numberofSimulations,signature,cancer_type,e
         if ((listofSimulationsSignatureBased is not None) and listofSimulationsSignatureBased):
             stackedSimulationsSignatureBased = np.vstack(listofSimulationsSignatureBased)
             (rows, cols) = stackedSimulationsSignatureBased.shape
-            print('After np.vstack --- stackedSimulationsSignatureBased rows:%d cols:%d' % (rows, cols))
+            if verbose: print('\tVerbose After np.vstack --- stackedSimulationsSignatureBased rows:%d cols:%d' % (rows, cols))
 
             #Get the part that we are interested
             stackedSimulationsSignatureBased_of_interest= stackedSimulationsSignatureBased[:,start:end]
             (rows, cols) = stackedSimulationsSignatureBased_of_interest.shape
-            print('After get part interested in --- stackedSimulationsSignatureBased_of_interest rows:%d cols:%d' % (rows, cols))
+            if verbose: print(
+                '\tVerbose After get part interested in --- stackedSimulationsSignatureBased_of_interest rows:%d cols:%d' % (rows, cols))
 
             # Get rid of all the rows with all nans
             stackedSimulationsSignatureBased_of_interest = stackedSimulationsSignatureBased_of_interest[~np.isnan(stackedSimulationsSignatureBased_of_interest).all(axis=1)]
             (rows, cols) = stackedSimulationsSignatureBased_of_interest.shape
-            print('After removing all nan rows --- stackedSimulationsSignatureBased_of_interest rows:%d cols:%d' % (rows, cols))
+            if verbose: print(
+                '\tVerbose After removing all nan rows --- stackedSimulationsSignatureBased_of_interest rows:%d cols:%d' % (rows, cols))
 
             simulationsSignatureBasedMeans = np.nanmean(stackedSimulationsSignatureBased_of_interest, axis=1)
-            avg_simulated_signal = np.nanmean(simulationsSignatureBasedMeans)
+            if not np.all(np.isnan(simulationsSignatureBasedMeans)):
+                avg_simulated_signal = np.nanmean(simulationsSignatureBasedMeans)
 
     if (avg_real_signal is not None) and (avg_simulated_signal is not None):
         fold_change = avg_real_signal / avg_simulated_signal
-        print('----------> %s %s %s avg_real_signal:%f\tavg_simulated_signal:%f\tfold change:%f' % (signature, cancer_type, encode_hm,avg_real_signal, avg_simulated_signal, fold_change))
+        if verbose: print('\tVerbose ----------> %s %s %s avg_real_signal:%f\tavg_simulated_signal:%f\tfold change:%f' % (signature, cancer_type, encode_hm,avg_real_signal, avg_simulated_signal, fold_change))
         return fold_change
     else:
-        print('----------> %s %s %s avg_real_signal:%s\tavg_simulated_signal:%s\tfold change: nan' % (signature,cancer_type,encode_hm,str(avg_real_signal),str(avg_simulated_signal)))
+        if verbose: print('\tVerbose ----------> %s %s %s avg_real_signal:%s\tavg_simulated_signal:%s\tfold change: nan' % (signature,cancer_type,encode_hm,str(avg_real_signal),str(avg_simulated_signal)))
         return None
 ########################################################
 
@@ -1010,7 +1011,7 @@ def calculate_fold_change(output_dir,numberofSimulations,signature,cancer_type,e
 
 ########################################################
 #Constraint epigenomic biosample name must be contained in epigenomic file memo
-def fill_signature2CancerType2Biosample2HM2FoldChangeDict(output_dir,numberofSimulations,signature,cancer_type,epigenomics_files_memos,epigenomics_biosamples,occupancy_type):
+def fill_signature2CancerType2Biosample2HM2FoldChangeDict(output_dir,numberofSimulations,signature,cancer_type,epigenomics_files_memos,epigenomics_biosamples,occupancy_type,plusOrMinus_epigenomics,verbose):
 
     list=[]
 
@@ -1021,26 +1022,26 @@ def fill_signature2CancerType2Biosample2HM2FoldChangeDict(output_dir,numberofSim
 
     ########################################################################
     #Step1
-    print('===================================================================================================')
-    print('signature:%s' %(signature))
+    if verbose: print('\tVerbose ===================================================================================================')
+    if verbose: print('\tVerbose signature:%s' %(signature))
 
     # epigenomics_files_memos have been used in naming
     # ID8_sim56_H3K27me3_Breast_Epithelium_AccumulatedCountArray.txt
     # ID8_sim56_H3K27me3_Breast_Epithelium_AccumulatedSignalArray.txt
 
     encode_hms = epigenomics_files_memos
-    print('#####################################################')
-    print('%s' %(encode_hms))
+    if verbose: print('\tVerbose #####################################################')
+    if verbose: print('\tVerbose %s' %(encode_hms))
 
     for encode_hm in encode_hms:
-        fold_change=calculate_fold_change(output_dir,numberofSimulations,signature,cancer_type,encode_hm)
+        fold_change=calculate_fold_change(output_dir,numberofSimulations,signature,cancer_type,encode_hm,plusOrMinus_epigenomics,verbose)
         if fold_change is not None:
             updateDictWithTwoLevels(signature2EncodeHM2FoldChangeDict,signature,encode_hm,fold_change)
 
-    print('#######################################################')
-    print('Step1 After %s' % (signature))
-    print('signature2EncodeHM2FoldChangeDict')
-    print(signature2EncodeHM2FoldChangeDict)
+    if verbose: print('\tVerbose #######################################################')
+    if verbose: print('\tVerbose Step1 After %s' % (signature))
+    if verbose: print('\tVerbose signature2EncodeHM2FoldChangeDict')
+    if verbose: print('\tVerbose %s' %signature2EncodeHM2FoldChangeDict)
     ########################################################################
 
     ########################################################################
@@ -1056,17 +1057,17 @@ def fill_signature2CancerType2Biosample2HM2FoldChangeDict(output_dir,numberofSim
                         fold_change=signature2EncodeHM2FoldChangeDict[signature][encode_hm]
                         updateDictWithThreeLevels(signature2Biosample2EncodeHM2FoldChangeDict, signature, biosample, encode_hm, fold_change)
 
-
-    print('#######################################################')
-    print('Step2 After %s' % (signature))
-    print('signature2Biosample2EncodeHM2FoldChangeDict')
-    print(signature2Biosample2EncodeHM2FoldChangeDict)
+    if verbose: print('\tVerbose #######################################################')
+    if verbose: print('\tVerbose Step2 After %s' % (signature))
+    if verbose: print('\tVerbose signature2Biosample2EncodeHM2FoldChangeDict')
+    if verbose: print('\tVerbose %s' %signature2Biosample2EncodeHM2FoldChangeDict)
     ########################################################################
 
     list.append(output_dir)
     list.append(cancer_type)
     list.append(occupancy_type)
     list.append(signature2Biosample2EncodeHM2FoldChangeDict)
+    list.append(verbose)
 
     return list
 ########################################################
@@ -1273,6 +1274,7 @@ def plot_pcawg_heatmap(list):
     cancer_type = list[1]
     occupancy_type=list[2]
     signature2Biosample2EncodeHM2FoldChangeDict=list[3]
+    verbose=list[4]
 
     os.makedirs(os.path.join(output_dir, cancer_type, FIGURE, ALL, occupancy_type,HEATMAPS), exist_ok=True)
     heatmap_output_path=os.path.join(output_dir, cancer_type, FIGURE, ALL, occupancy_type,HEATMAPS)
@@ -1282,7 +1284,7 @@ def plot_pcawg_heatmap(list):
         for biosample in signature2Biosample2EncodeHM2FoldChangeDict[signature]:
             encode_hms, average_fold_change_array = fill_average_fold_change_array_signature_cancertype_biosample(signature2Biosample2EncodeHM2FoldChangeDict[signature][biosample])
 
-            print('len(encode_hms):%d %s' %(len(encode_hms),encode_hms))
+            if verbose: print('\tVerbose len(encode_hms):%d %s' %(len(encode_hms),encode_hms))
             cancer_type_encode_biosamples_list=[]
             cancer_type_encode_biosamples_list.append('%s, %s' %(cancer_type,biosample))
 
@@ -1291,13 +1293,13 @@ def plot_pcawg_heatmap(list):
             fig, ax = plt.subplots(figsize=(100, 40))
 
             # im, cbar = heatmap(fold_change_array, signatures, hms, ax=ax,cmap="seismic", cbarlabel="Fold Change [real/simulated]")
-            print('min:%f max:%f' %(np.min(average_fold_change_array),np.max(average_fold_change_array)))
+            if verbose: print('\tVerbose min:%f max:%f' %(np.min(average_fold_change_array),np.max(average_fold_change_array)))
 
             #Blue White Red
             im, cbar = heatmap(average_fold_change_array, cancer_type_encode_biosamples_list, encode_hms, ax=ax, cmap='seismic', cbarlabel="Fold Change [Real mutations/Simulated Mutations]",vmin=0, vmax=2)
 
             texts = annotate_heatmap(im, valfmt="{x:.2f} ", fontsize=50)
-            print('texts:%s' %texts)
+            if verbose: print('\tVerbose texts:%s' %texts)
 
             plt.title('%s' % (signature), fontsize=50, y=1.01)
             #Results in big squares when array is small
@@ -1305,6 +1307,9 @@ def plot_pcawg_heatmap(list):
             filename = '%s_rows_%s_%s_columns_encode_hms_heatmap.png' % (signature,cancer_type,biosample)
             figureFile = os.path.join(heatmap_output_path,filename)
 
+            # Results in big squares when array is small
+            plt.tight_layout()
+
             fig.savefig(figureFile)
             plt.clf()
             plt.cla()
@@ -1317,7 +1322,7 @@ def plot_pcawg_heatmap(list):
 
 
 #########################################################
-def plot_heatmaps(outputDir,jobname,numberofSimulations,epigenomics_files_memos,epigenomics_biosamples,occupancy_type):
+def plot_heatmaps(outputDir,jobname,numberofSimulations,epigenomics_files_memos,epigenomics_biosamples,occupancy_type,plusOrMinus_epigenomics,verbose):
 
     #We can plot heatmaps if there are at least one simulation
     if (numberofSimulations>=1):
@@ -1346,7 +1351,7 @@ def plot_heatmaps(outputDir,jobname,numberofSimulations,epigenomics_files_memos,
         pool = multiprocessing.Pool(numofProcesses)
 
         for signature in signatures:
-            pool.apply_async(fill_signature2CancerType2Biosample2HM2FoldChangeDict, (outputDir,numberofSimulations,signature,cancer_type,epigenomics_files_memos,epigenomics_biosamples,occupancy_type), callback=plot_pcawg_heatmap)
+            pool.apply_async(fill_signature2CancerType2Biosample2HM2FoldChangeDict, args=(outputDir,numberofSimulations,signature,cancer_type,epigenomics_files_memos,epigenomics_biosamples,occupancy_type,plusOrMinus_epigenomics,verbose,), callback=plot_pcawg_heatmap)
             #Sequential Run
             # list=fill_signature2CancerType2Biosample2HM2FoldChangeDict(outputDir,numberofSimulations,signature,cancer_type,epigenomics_files_memos,epigenomics_biosamples,occupancy_type)
             # plot_pcawg_heatmap(list)
@@ -1376,7 +1381,6 @@ def occupancyAverageSignalFigures(outputDir,jobname,figureAugmentation,numberofS
         isFigureAugmentation = True
 
     ############## Read necessary dictionaries starts ########################################
-
     mutationtype_numberofmutations_df = pd.read_csv(os.path.join(outputDir, jobname, DATA,Table_MutationType_NumberofMutations_Filename),sep='\t', header=0,dtype={'mutation_type': str,'number_of_mutations': np.int32})
     subsSignature_cutoff_numberofmutations_averageprobability_df = pd.read_csv(os.path.join(outputDir, jobname, DATA,Table_SubsSignature_Cutoff_NumberofMutations_AverageProbability_Filename),sep='\t', header=0,dtype={'cutoff': np.float32,'signature': str,'number_of_mutations': np.int32,'average_probability': np.float32})
     indelsSignature_cutoff_numberofmutations_averageprobability_df = pd.read_csv(os.path.join(outputDir, jobname, DATA,Table_IndelsSignature_Cutoff_NumberofMutations_AverageProbability_Filename),sep='\t', header=0,dtype={'cutoff': np.float32,'signature': str,'number_of_mutations': np.int32,'average_probability': np.float32})
